// Main.cc
// Simple TK main

#include <tk.h>
#include <iostream.h>
#include <strstream.h>
#include <string.h>
#include "pad.h"
#include "defs.h"

int SolverLogStep();
int SolverInit(int argc, char ** argv );

int PadCmd ( ClientData clientData, Tcl_Interp * interp,
	int argc, char ** argv );

int DoOneLog ( ClientData clientData, Tcl_Interp * interp,
	int argc, char ** argv );

int DoAppSetup ( ClientData clientData, Tcl_Interp * interp,
	int argc, char ** argv );

int Tcl_AppInit ( Tcl_Interp * interp );

int voptions(int argc, char** argv);
int vmain(int argc, char ** argv );

int main ( int argc, char **argv )
{
	int i = voptions(argc, argv) - 1;
	cout << "VOptions returns " << i << endl;
	cout << "argc = " << argc << endl;
	Tk_Main ( argc - i, &argv[i], Tcl_AppInit );
	return 0;
}

extern char TclSrc[];

int Tcl_AppInit ( Tcl_Interp * interp )
{
	if ( Tcl_Init ( interp ) == TCL_ERROR )
		return TCL_ERROR;
	if ( Tk_Init ( interp ) == TCL_ERROR )
		return TCL_ERROR;

	Tcl_CreateCommand ( interp, "pad", PadCmd,
		(ClientData) Tk_MainWindow ( interp ),
		(Tcl_CmdDeleteProc *) NULL );

	Tcl_CreateCommand ( interp, "doOneLog", DoOneLog,
		(ClientData) Tk_MainWindow ( interp ),
		(Tcl_CmdDeleteProc *) NULL );

	Tcl_CreateCommand ( interp, "doAppSetup", DoAppSetup,
		(ClientData) Tk_MainWindow ( interp ),
		(Tcl_CmdDeleteProc *) NULL );
	//tcl procedures store there answers in a string
	//The delete procedure says how to delete this string
	// NULL means the string does not need to be dealloc
	// (it is global or static)

	Tcl_Eval(interp, TclSrc );
	//Tcl_Eval(interp, "source solver.tk" );
	//tcl_RcFileName = "~/.solverrc.tcl";
	return TCL_OK;
}

Pad * padPtr;

int DoOneLog ( ClientData clientData, Tcl_Interp * interp,
	int argc, char ** argv )
{
	const int MyAnswerLen = 20; // could use the default buffer.
	static char MyAnswer[MyAnswerLen];
	ostrstream MyOut( MyAnswer, MyAnswerLen );
	int i;

	//cout << "DoOneLog Called argc =" << argc << endl;
	//for ( i = 0; i < argc; i++ )
	//	cout << "i is " << i << " argv is " << argv[i] << endl;

	if ( ! SolverLogStep() )
	{
		//cout << "No More Logs" << endl;
		#if SFB
		Tcl_Eval(interp, "doStop");
		//Tcl_Eval(interp, "after idle {destroy .}" );
		#else
		Tcl_Eval(interp, "doStop");
		Tcl_Eval(interp, "after idle {destroy .}" );
		#endif
		voronoiEnd();
		PadFlush ( padPtr );
	}

	MyOut << argc << ends;
	interp->result = MyAnswer;
	return TCL_OK;
}

int Debug ( ClientData clientData, Tcl_Interp * interp,
	int argc, char ** argv )
{
	char line[1000];

	cout << "debug% ";
	cout << flush;
	while ( gets(line) )
	{
		if ( strcmp (line, "quit" ) == 0 )
			break;
		Tcl_Eval(interp, line);
		cout << interp->result << endl;
		cout << "debug% ";
		cout << flush;
	}
	return TCL_OK;
}


// This translates its paramater ".pad" in this case
// to the global padPtr.

int DoAppSetup ( ClientData clientData, Tcl_Interp * interp,
	int argc, char ** argv )
{
	int i;

	//for ( i = 0; i < argc; i++ )
	//	cout << "arg[" << i << "] is " << argv[i] << endl;
	if ( argc != 2 )
	{
		Tcl_AppendResult(interp, "wrong # args:  should be \"",
			    argv[0], " pathName \"", (char *) NULL);
		return TCL_ERROR;
	}

	Tcl_CmdInfo Cmd;

	if ( Tcl_GetCommandInfo ( interp, argv[1], &Cmd ) == 1 )
	{
		padPtr = (Pad *) Cmd.clientData;
	}
	SolverInit( 0, (char **) NULL );
	return TCL_OK;
}

int x = 1;
int y = 1;

int SolverLogStep()
{
	int answer = voronoiLoop();
	PadFlush( padPtr );
	return answer;
}

int SolverInit(int argc, char ** argv )
{
	PadFgColor ( padPtr, PadGetColor ( padPtr, "red" ) );
	PadClear ( padPtr );
	// argc and argv are not correct here.
	// what needs to be done?
	// here argc = 1 so things are ok, but there is no way to pass
	// parameters?
	vmain ( 0, NULL );
	return 0;
}
